@@ -56,7 +56,7 @@ from .ast import (
 )
 from .directive_locations import DirectiveLocation
 from .ast import Token
-from .lexer import Lexer
+from .lexer import Lexer, is_punctuator_token_kind
 from .source import Source
 from .token_kind import TokenKind
 from ..error import GraphQLError, GraphQLSyntaxError
@@ -77,14 +77,16 @@ def parse(
     Throws GraphQLError if a syntax error is encountered.
 
     By default, the parser creates AST nodes that know the location in the source that
-    they correspond to. The `no_location` option disables that behavior for performance
-    or testing.
+    they correspond to. The ``no_location`` option disables that behavior for
+    performance or testing.
 
     Experimental features:
 
-    If `experimental_fragment_variables` is set to True, the parser will understand
-    and parse variable definitions contained in a fragment definition. They'll be
-    represented in the `variable_definitions` field of the `FragmentDefinitionNode`.
+    If ``experimental_fragment_variables`` is set to ``True``, the parser will
+    understand and parse variable definitions contained in a fragment definition.
+    They'll be represented in the
+    :attr:`~graphql.language.FragmentDefinitionNode.variable_definitions` field
+    of the :class:`~graphql.language.FragmentDefinitionNode`.
 
     The syntax is identical to normal, query-defined variables. For example::
 
@@ -110,7 +112,8 @@ def parse_value(
     This is useful within tools that operate upon GraphQL Values directly and in
     isolation of complete GraphQL documents.
 
-    Consider providing the results to the utility function: `value_from_ast()`.
+    Consider providing the results to the utility function:
+    :func:`~graphql.value_from_ast`.
     """
     parser = Parser(
         source,
@@ -133,7 +136,8 @@ def parse_type(
     This is useful within tools that operate upon GraphQL Types directly and
     in isolation of complete GraphQL documents.
 
-    Consider providing the results to the utility function: `type_from_ast()`.
+    Consider providing the results to the utility function:
+    :func:`~graphql.value_from_ast`.
     """
     parser = Parser(
         source,
@@ -161,7 +165,7 @@ class Parser:
         if isinstance(source, str):
             source = Source(source)
         elif not isinstance(source, Source):
-            raise TypeError(f"Must provide Source. Received: {inspect(source)}")
+            raise TypeError(f"Must provide Source. Received: {inspect(source)}.")
         self._lexer = Lexer(source)
         self._no_location = no_location
         self._experimental_fragment_variables = experimental_fragment_variables
@@ -390,7 +394,7 @@ class Parser:
         )
 
     def parse_fragment_name(self) -> NameNode:
-        """FragmentName: Name but not `on`"""
+        """FragmentName: Name but not ``on``"""
         if self._lexer.token.value == "on":
             raise self.unexpected()
         return self.parse_name()
@@ -415,9 +419,9 @@ class Parser:
 
     def parse_value_literal(self, is_const: bool) -> ValueNode:
         method_name = self._parse_value_literal_method_names.get(self._lexer.token.kind)
-        if method_name:
+        if method_name:  # pragma: no cover
             return getattr(self, f"parse_{method_name}")(is_const)
-        raise self.unexpected()
+        raise self.unexpected()  # pragma: no cover
 
     def parse_string_literal(self, _is_const: bool = False) -> StringValueNode:
         token = self._lexer.token
@@ -470,12 +474,13 @@ class Parser:
         token = self._lexer.token
         value = token.value
         self._lexer.advance()
-        if value in ("true", "false"):
-            return BooleanValueNode(value=value == "true", loc=self.loc(token))
-        elif value == "null":
+        if value == "true":
+            return BooleanValueNode(value=True, loc=self.loc(token))
+        if value == "false":
+            return BooleanValueNode(value=False, loc=self.loc(token))
+        if value == "null":
             return NullValueNode(loc=self.loc(token))
-        else:
-            return EnumValueNode(value=value, loc=self.loc(token))
+        return EnumValueNode(value=value, loc=self.loc(token))
 
     def parse_variable_value(self, is_const: bool) -> VariableNode:
         if not is_const:
@@ -565,7 +570,7 @@ class Parser:
             method_name = self._parse_type_extension_method_names.get(
                 cast(str, keyword_token.value)
             )
-            if method_name:
+            if method_name:  # pragma: no cover
                 return getattr(self, f"parse_{method_name}")()
         raise self.unexpected(keyword_token)
 
@@ -581,13 +586,17 @@ class Parser:
     def parse_schema_definition(self) -> SchemaDefinitionNode:
         """SchemaDefinition"""
         start = self._lexer.token
+        description = self.parse_description()
         self.expect_keyword("schema")
         directives = self.parse_directives(True)
         operation_types = self.many(
             TokenKind.BRACE_L, self.parse_operation_type_definition, TokenKind.BRACE_R
         )
         return SchemaDefinitionNode(
-            directives=directives, operation_types=operation_types, loc=self.loc(start)
+            description=description,
+            directives=directives,
+            operation_types=operation_types,
+            loc=self.loc(start),
         )
 
     def parse_operation_type_definition(self) -> OperationTypeDefinitionNode:
@@ -703,11 +712,13 @@ class Parser:
         description = self.parse_description()
         self.expect_keyword("interface")
         name = self.parse_name()
+        interfaces = self.parse_implements_interfaces()
         directives = self.parse_directives(True)
         fields = self.parse_fields_definition()
         return InterfaceTypeDefinitionNode(
             description=description,
             name=name,
+            interfaces=interfaces,
             directives=directives,
             fields=fields,
             loc=self.loc(start),
@@ -852,12 +863,17 @@ class Parser:
         self.expect_keyword("extend")
         self.expect_keyword("interface")
         name = self.parse_name()
+        interfaces = self.parse_implements_interfaces()
         directives = self.parse_directives(True)
         fields = self.parse_fields_definition()
-        if not (directives or fields):
+        if not (interfaces or directives or fields):
             raise self.unexpected()
         return InterfaceTypeExtensionNode(
-            name=name, directives=directives, fields=fields, loc=self.loc(start)
+            name=name,
+            interfaces=interfaces,
+            directives=directives,
+            fields=fields,
+            loc=self.loc(start),
         )
 
     def parse_union_type_extension(self) -> UnionTypeExtensionNode:
@@ -952,9 +968,7 @@ class Parser:
         if not self._no_location:
             end_token = self._lexer.last_token
             source = self._lexer.source
-            return Location(
-                start_token.start, end_token.end, start_token, end_token, source
-            )
+            return Location(start_token, end_token, source)
         return None
 
     def peek(self, kind: TokenKind) -> bool:
@@ -975,7 +989,7 @@ class Parser:
         raise GraphQLSyntaxError(
             self._lexer.source,
             token.start,
-            f"Expected {kind.value}, found {token.kind.value}",
+            f"Expected {get_token_kind_desc(kind)}, found {get_token_desc(token)}.",
         )
 
     def expect_optional_token(self, kind: TokenKind) -> Optional[Token]:
@@ -1004,7 +1018,7 @@ class Parser:
             raise GraphQLSyntaxError(
                 self._lexer.source,
                 token.start,
-                f"Expected {value!r}, found {token.desc}",
+                f"Expected '{value}', found {get_token_desc(token)}.",
             )
 
     def expect_optional_keyword(self, value: str) -> bool:
@@ -1020,11 +1034,11 @@ class Parser:
 
         return False
 
-    def unexpected(self, at_token: Token = None) -> GraphQLError:
+    def unexpected(self, at_token: Optional[Token] = None) -> GraphQLError:
         """Create an error when an unexpected lexed token is encountered."""
         token = at_token or self._lexer.token
         return GraphQLSyntaxError(
-            self._lexer.source, token.start, f"Unexpected {token.desc}"
+            self._lexer.source, token.start, f"Unexpected {get_token_desc(token)}."
         )
 
     def any(
@@ -1032,9 +1046,10 @@ class Parser:
     ) -> List[T]:
         """Fetch any matching nodes, possibly none.
 
-        Returns a possibly empty list of parse nodes, determined by the `parse_fn`.
-        This list begins with a lex token of `open_kind` and ends with a lex token of
-        `close_kind`. Advances the parser to the next lex token after the closing token.
+        Returns a possibly empty list of parse nodes, determined by the ``parse_fn``.
+        This list begins with a lex token of ``open_kind`` and ends with a lex token of
+        ``close_kind``. Advances the parser to the next lex token after the closing
+        token.
         """
         self.expect_token(open_kind)
         nodes: List[T] = []
@@ -1048,10 +1063,11 @@ class Parser:
     ) -> List[T]:
         """Fetch matching nodes, maybe none.
 
-        Returns a list of parse nodes, determined by the `parse_fn`. It can be empty
+        Returns a list of parse nodes, determined by the ``parse_fn``. It can be empty
         only if the open token is missing, otherwise it will always return a non-empty
-        list that begins with a lex token of `open_kind` and ends with a lex token of
-        `close_kind`. Advances the parser to the next lex token after the closing token.
+        list that begins with a lex token of ``open_kind`` and ends with a lex token of
+        ``close_kind``. Advances the parser to the next lex token after the closing
+        token.
         """
         if self.expect_optional_token(open_kind):
             nodes = [parse_fn()]
@@ -1066,9 +1082,10 @@ class Parser:
     ) -> List[T]:
         """Fetch matching nodes, at least one.
 
-        Returns a non-empty list of parse nodes, determined by the `parse_fn`.
-        This list begins with a lex token of `open_kind` and ends with a lex token of
-        `close_kind`. Advances the parser to the next lex token after the closing token.
+        Returns a non-empty list of parse nodes, determined by the ``parse_fn``.
+        This list begins with a lex token of ``open_kind`` and ends with a lex token of
+        ``close_kind``. Advances the parser to the next lex token after the closing
+        token.
         """
         self.expect_token(open_kind)
         nodes = [parse_fn()]
@@ -1076,3 +1093,16 @@ class Parser:
         while not self.expect_optional_token(close_kind):
             append(parse_fn())
         return nodes
+
+
+def get_token_desc(token: Token) -> str:
+    """Describe a token as a string for debugging."""
+    value = token.value
+    return get_token_kind_desc(token.kind) + (
+        f" '{value}'" if value is not None else ""
+    )
+
+
+def get_token_kind_desc(kind: TokenKind) -> str:
+    """Describe a token kind as a string for debugging."""
+    return f"'{kind.value}'" if is_punctuator_token_kind(kind) else kind.value
