@@ -37,11 +37,11 @@ def get_response(test_type, test_data):
         },
     )
 
-    schema = GraphQLSchema(data_type)
-
-    ast = parse("{ nest { test } }")
-
-    return execute(schema, ast, data)
+    return execute(
+        schema=GraphQLSchema(data_type),
+        document=parse("{ nest { test } }"),
+        context_value=data,
+    )
 
 
 def check_response(response, expected):
@@ -106,7 +106,7 @@ def describe_execute_accepts_any_iterable_as_list_value():
                 [
                     {
                         "message": "Expected Iterable,"
-                        " but did not find one for field DataType.test.",
+                        " but did not find one for field 'DataType.test'.",
                         "locations": [(1, 10)],
                         "path": ["nest", "test"],
                     }
