@@ -96,9 +96,10 @@ schema = build_schema(
 
 
 def execute_query(query, root_value):
-    return execute(schema, parse(query), root_value)
+    return execute(schema=schema, document=parse(query), root_value=root_value)
 
 
+# avoids also doing any nests
 def patch(data):
     return re.sub(
         r"\bsyncNonNull\b", "promiseNonNull", re.sub(r"\bsync\b", "promise", data)
@@ -538,10 +539,6 @@ def describe_execute_handles_non_nullable_types():
     def describe_handles_non_null_argument():
 
         # noinspection PyPep8Naming
-        def _resolve(_obj, _info, cannotBeNull):
-            if isinstance(cannotBeNull, str):
-                return f"Passed: {cannotBeNull}"
-
         schema_with_non_null_arg = GraphQLSchema(
             GraphQLObjectType(
                 "Query",
@@ -553,7 +550,8 @@ def describe_execute_handles_non_nullable_types():
                                 GraphQLNonNull(GraphQLString)
                             )
                         },
-                        resolve=_resolve,
+                        resolve=lambda _obj, _info, cannotBeNull: "Passed: "
+                        + str(cannotBeNull),
                     )
                 },
             )
