@@ -1,9 +1,9 @@
 from math import nan
 
-from graphql.error import INVALID
 from graphql.execution import execute
 from graphql.execution.values import get_variable_values
 from graphql.language import parse, OperationDefinitionNode
+from graphql.pyutils import Undefined
 from graphql.type import (
     GraphQLArgument,
     GraphQLEnumType,
@@ -20,15 +20,16 @@ from graphql.type import (
     GraphQLString,
 )
 
+
+def parse_serialized_value(value: str) -> str:
+    assert value == "SerializedValue"
+    return "DeserializedValue"
+
+
 TestComplexScalar = GraphQLScalarType(
     name="ComplexScalar",
-    serialize=lambda value: "SerializedValue" if value == "DeserializedValue" else None,
-    parse_value=lambda value: "DeserializedValue"
-    if value == "SerializedValue"
-    else None,
-    parse_literal=lambda ast, _variables=None: "DeserializedValue"
-    if ast.value == "SerializedValue"
-    else None,
+    parse_value=lambda value: parse_serialized_value(value),
+    parse_literal=lambda ast, _variables=None: parse_serialized_value(ast.value),
 )
 
 
@@ -62,7 +63,7 @@ TestEnum = GraphQLEnumType(
     "TestEnum",
     {
         "NULL": None,
-        "UNDEFINED": INVALID,
+        "UNDEFINED": Undefined,
         "NAN": nan,
         "FALSE": False,
         "CUSTOM": "custom value",
@@ -365,7 +366,7 @@ def describe_execute_handles_inputs():
                         {
                             "message": "Variable '$input' got invalid value"
                             " None at 'input.c';"
-                            " Expected non-nullable type String! not to be None.",
+                            " Expected non-nullable type 'String!' not to be None.",
                             "locations": [(2, 24)],
                         }
                     ],
@@ -379,7 +380,7 @@ def describe_execute_handles_inputs():
                     [
                         {
                             "message": "Variable '$input' got invalid value 'foo bar';"
-                            " Expected type TestInputObject to be a dict.",
+                            " Expected type 'TestInputObject' to be a dict.",
                             "locations": [(2, 24)],
                             "path": None,
                         }
@@ -395,7 +396,7 @@ def describe_execute_handles_inputs():
                         {
                             "message": "Variable '$input' got invalid value"
                             " {'a': 'foo', 'b': 'bar'};"
-                            " Field c of required type String! was not provided.",
+                            " Field 'c' of required type 'String!' was not provided.",
                             "locations": [(2, 24)],
                         }
                     ],
@@ -415,13 +416,13 @@ def describe_execute_handles_inputs():
                         {
                             "message": "Variable '$input' got invalid value"
                             " {'a': 'foo'} at 'input.na';"
-                            " Field c of required type String! was not provided.",
+                            " Field 'c' of required type 'String!' was not provided.",
                             "locations": [(2, 28)],
                         },
                         {
                             "message": "Variable '$input' got invalid value"
                             " {'na': {'a': 'foo'}};"
-                            " Field nb of required type String! was not provided.",
+                            " Field 'nb' of required type 'String!' was not provided.",
                             "locations": [(2, 28)],
                         },
                     ],
@@ -437,7 +438,7 @@ def describe_execute_handles_inputs():
                         {
                             "message": "Variable '$input' got invalid value {'a':"
                             " 'foo', 'b': 'bar', 'c': 'baz', 'extra': 'dog'}; Field"
-                            " 'extra' is not defined by type TestInputObject.",
+                            " 'extra' is not defined by type 'TestInputObject'.",
                             "locations": [(2, 24)],
                         }
                     ],
@@ -658,15 +659,14 @@ def describe_execute_handles_inputs():
                 [
                     {
                         "message": "Variable '$value' got invalid value [1, 2, 3];"
-                        " Expected type String. String cannot represent"
-                        " a non string value: [1, 2, 3]",
+                        " String cannot represent a non string value: [1, 2, 3]",
                         "locations": [(2, 24)],
                         "path": None,
                     }
                 ],
             )
 
-            assert isinstance(result.errors[0].original_error, TypeError)
+            assert result.errors[0].original_error is None
 
         def reports_error_for_non_provided_variables_for_non_nullable_inputs():
             # Note: this test would typically fail validation before
@@ -807,7 +807,7 @@ def describe_execute_handles_inputs():
                     {
                         "message": "Variable '$input' got invalid value None"
                         " at 'input[1]';"
-                        " Expected non-nullable type String! not to be None.",
+                        " Expected non-nullable type 'String!' not to be None.",
                         "locations": [(2, 24)],
                     }
                 ],
@@ -856,7 +856,7 @@ def describe_execute_handles_inputs():
                     {
                         "message": "Variable '$input' got invalid value None"
                         " at 'input[1]';"
-                        " Expected non-nullable type String! not to be None.",
+                        " Expected non-nullable type 'String!' not to be None.",
                         "locations": [(2, 24)],
                         "path": None,
                     }
@@ -889,7 +889,7 @@ def describe_execute_handles_inputs():
                   fieldWithObjectInput(input: $input)
                 }
                 """
-            result = execute_query(doc, {"input": "whoknows"})
+            result = execute_query(doc, {"input": "whoKnows"})
 
             assert result == (
                 None,
@@ -974,11 +974,19 @@ def describe_execute_handles_inputs():
             return {
                 "message": "Variable '$input' got invalid value"
                 f" {value} at 'input[{index}]';"
-                " Expected type String."
                 f" String cannot represent a non string value: {value}",
                 "locations": [(2, 20)],
             }
 
+        def return_all_errors_by_default():
+            result = get_variable_values(schema, variable_definitions, input_value)
+
+            assert result == [
+                _invalid_value_error(0, 0),
+                _invalid_value_error(1, 1),
+                _invalid_value_error(2, 2),
+            ]
+
         def when_max_errors_is_equal_to_number_of_errors():
             result = get_variable_values(
                 schema, variable_definitions, input_value, max_errors=3
