@@ -1,27 +1,25 @@
 from copy import copy
 from functools import partial
-from typing import List
+from typing import cast, Dict, List, Optional, Tuple
+
+from pytest import raises  # type: ignore
 
 from graphql.language import (
     Node,
     FieldNode,
     NameNode,
+    SelectionNode,
     SelectionSetNode,
     parse,
-    print_ast,
     visit,
     BREAK,
     REMOVE,
     SKIP,
     ParallelVisitor,
-    TypeInfoVisitor,
     Visitor,
 )
+from graphql.language.visitor import QUERY_DOCUMENT_KEYS
 from graphql.pyutils import FrozenList
-from graphql.type import get_named_type, is_composite_type
-from graphql.utilities import TypeInfo
-
-from ..validation.harness import test_schema
 
 # noinspection PyUnresolvedReferences
 from ..fixtures import kitchen_sink_query  # noqa: F401
@@ -86,6 +84,24 @@ def get_value(node):
 
 
 def describe_visitor():
+    def visit_with_invalid_node():
+        with raises(TypeError) as exc_info:
+            # noinspection PyTypeChecker
+            visit("invalid", Visitor())  # type: ignore
+        assert str(exc_info.value) == "Not an AST Node: 'invalid'."
+
+    def visit_with_invalid_visitor():
+        ast = parse("{ a }", no_location=True)
+
+        class TestVisitor:
+            def enter(self, *_args):
+                pass
+
+        with raises(TypeError) as exc_info:
+            # noinspection PyTypeChecker
+            visit(ast, TestVisitor())  # type: ignore
+        assert str(exc_info.value) == "Not an AST Visitor: <TestVisitor instance>."
+
     def validates_path_argument():
         ast = parse("{ a }", no_location=True)
         visited = []
@@ -138,6 +154,23 @@ def describe_visitor():
 
         visit(ast, TestVisitor())
 
+    def allows_visiting_only_specified_nodes():
+        ast = parse("{ a }", no_location=True)
+        visited = []
+
+        # noinspection PyMethodMayBeStatic
+        class TestVisitor(Visitor):
+            selection_set = None
+
+            def enter_field(self, node, *_args):
+                visited.append(["enter", node.kind])
+
+            def leave_field(self, node, *_args):
+                visited.append(["leave", node.kind])
+
+        visit(ast, TestVisitor())
+        assert visited == [["enter", "field"], ["leave", "field"]]
+
     def allows_editing_a_node_both_on_enter_and_on_leave():
         ast = parse("{ a, b, c { a, b, c } }", no_location=True)
         visited = []
@@ -196,6 +229,19 @@ def describe_visitor():
         assert ast == parse("{ a, b, c { a, b, c } }", no_location=True)
         assert edited_ast == parse("{ a,    c { a,    c } }", no_location=True)
 
+    def ignores_false_returned_on_leave():
+        ast = parse("{ a, b, c { a, b, c } }", no_location=True)
+
+        assert SKIP is False
+
+        # noinspection PyMethodMayBeStatic
+        class TestVisitor(Visitor):
+            def leave(self, *args):
+                return SKIP
+
+        returned_ast = visit(ast, TestVisitor())
+        assert returned_ast == parse("{ a, b, c { a, b, c } }", no_location=True)
+
     def visits_edited_node():
         ast = parse("{ a { x } }", no_location=True)
         added_field = FieldNode(name=NameNode(value="__typename"))
@@ -799,6 +845,113 @@ def describe_visitor():
         ]
 
 
+def describe_support_for_custom_ast_nodes():
+    custom_ast = parse("{ a }")
+
+    class CustomFieldNode(SelectionNode):
+        __slots__ = "name", "selection_set"
+
+        name: NameNode
+        selection_set: Optional[SelectionSetNode]
+
+    custom_selection_set = cast(FieldNode, custom_ast.definitions[0]).selection_set
+    assert custom_selection_set is not None
+    custom_selection_set.selections = custom_selection_set.selections + [
+        CustomFieldNode(
+            name=NameNode(value="b"),
+            selection_set=SelectionSetNode(
+                selections=CustomFieldNode(name=NameNode(value="c"))
+            ),
+        )
+    ]
+
+    def does_not_traverse_unknown_node_kinds():
+        visited = []
+
+        # noinspection PyMethodMayBeStatic
+        class TestVisitor(Visitor):
+            def enter(self, node, *_args):
+                visited.append(["enter", node.kind, get_value(node)])
+
+            def leave(self, node, *_args):
+                visited.append(["leave", node.kind, get_value(node)])
+
+        visit(custom_ast, TestVisitor())
+        assert visited == [
+            ["enter", "document", None],
+            ["enter", "operation_definition", None],
+            ["enter", "selection_set", None],
+            ["enter", "field", None],
+            ["enter", "name", "a"],
+            ["leave", "name", "a"],
+            ["leave", "field", None],
+            ["enter", "custom_field", None],
+            ["leave", "custom_field", None],
+            ["leave", "selection_set", None],
+            ["leave", "operation_definition", None],
+            ["leave", "document", None],
+        ]
+
+    def does_traverse_unknown_node_with_visitor_keys():
+        custom_query_document_keys: Dict[str, Tuple[str, ...]] = {
+            **QUERY_DOCUMENT_KEYS,
+            "custom_field": ("name", "selection_set"),
+        }
+        visited = []
+
+        # noinspection PyMethodMayBeStatic
+        class TestVisitor(Visitor):
+            def enter(self, node, *_args):
+                visited.append(["enter", node.kind, get_value(node)])
+
+            def leave(self, node, *_args):
+                visited.append(["leave", node.kind, get_value(node)])
+
+        visit(custom_ast, TestVisitor(), custom_query_document_keys)
+        assert visited == [
+            ["enter", "document", None],
+            ["enter", "operation_definition", None],
+            ["enter", "selection_set", None],
+            ["enter", "field", None],
+            ["enter", "name", "a"],
+            ["leave", "name", "a"],
+            ["leave", "field", None],
+            ["enter", "custom_field", None],
+            ["enter", "name", "b"],
+            ["leave", "name", "b"],
+            ["enter", "selection_set", None],
+            ["enter", "custom_field", None],
+            ["enter", "name", "c"],
+            ["leave", "name", "c"],
+            ["leave", "custom_field", None],
+            ["leave", "selection_set", None],
+            ["leave", "custom_field", None],
+            ["leave", "selection_set", None],
+            ["leave", "operation_definition", None],
+            ["leave", "document", None],
+        ]
+
+    def cannot_define_visitor_with_unknown_ast_nodes():
+        with raises(TypeError) as exc_info:
+
+            class VisitorWithNonExistingNode(Visitor):
+                def enter_field(self, *_args):
+                    pass
+
+                def leave_garfield(self, *_args):
+                    pass
+
+        assert str(exc_info.value) == "Invalid AST node kind: garfield."
+
+        with raises(TypeError) as exc_info:
+
+            class VisitorWithUnspecificNode(Visitor):
+                def enter_type_system_extension(self, *_args):
+                    pass
+
+        assert str(exc_info.value) == "Invalid AST node kind: type_system_extension."
+
+
 def describe_visit_in_parallel():
     def allows_skipping_a_sub_tree():
         # Note: nearly identical to the above test but using ParallelVisitor
@@ -958,6 +1111,7 @@ def describe_visit_in_parallel():
                     return BREAK
 
             def leave(self, *args):
+                assert self.name == "b"
                 check_visitor_fn_args(ast, *args)
                 node = args[0]
                 kind, value = node.kind, get_value(node)
@@ -1208,207 +1362,3 @@ def describe_visit_in_parallel():
             ["leave", "operation_definition", None],
             ["leave", "document", None],
         ]
-
-
-def describe_visit_with_type_info():
-    def maintains_type_info_during_visit():
-        visited = []
-
-        ast = parse("{ human(id: 4) { name, pets { ... { name } }, unknown } }")
-
-        type_info = TypeInfo(test_schema)
-
-        # noinspection PyMethodMayBeStatic
-        class TestVisitor(Visitor):
-            def enter(self, *args):
-                check_visitor_fn_args(ast, *args)
-                parent_type = type_info.get_parent_type()
-                type_ = type_info.get_type()
-                input_type = type_info.get_input_type()
-                node = args[0]
-                visited.append(
-                    [
-                        "enter",
-                        node.kind,
-                        node.value if node.kind == "name" else None,
-                        str(parent_type) if parent_type else None,
-                        str(type_) if type_ else None,
-                        str(input_type) if input_type else None,
-                    ]
-                )
-
-            def leave(self, *args):
-                check_visitor_fn_args(ast, *args)
-                parent_type = type_info.get_parent_type()
-                type_ = type_info.get_type()
-                input_type = type_info.get_input_type()
-                node = args[0]
-                visited.append(
-                    [
-                        "leave",
-                        node.kind,
-                        node.value if node.kind == "name" else None,
-                        str(parent_type) if parent_type else None,
-                        str(type_) if type_ else None,
-                        str(input_type) if input_type else None,
-                    ]
-                )
-
-        visit(ast, TypeInfoVisitor(type_info, TestVisitor()))
-
-        assert visited == [
-            ["enter", "document", None, None, None, None],
-            ["enter", "operation_definition", None, None, "QueryRoot", None],
-            ["enter", "selection_set", None, "QueryRoot", "QueryRoot", None],
-            ["enter", "field", None, "QueryRoot", "Human", None],
-            ["enter", "name", "human", "QueryRoot", "Human", None],
-            ["leave", "name", "human", "QueryRoot", "Human", None],
-            ["enter", "argument", None, "QueryRoot", "Human", "ID"],
-            ["enter", "name", "id", "QueryRoot", "Human", "ID"],
-            ["leave", "name", "id", "QueryRoot", "Human", "ID"],
-            ["enter", "int_value", None, "QueryRoot", "Human", "ID"],
-            ["leave", "int_value", None, "QueryRoot", "Human", "ID"],
-            ["leave", "argument", None, "QueryRoot", "Human", "ID"],
-            ["enter", "selection_set", None, "Human", "Human", None],
-            ["enter", "field", None, "Human", "String", None],
-            ["enter", "name", "name", "Human", "String", None],
-            ["leave", "name", "name", "Human", "String", None],
-            ["leave", "field", None, "Human", "String", None],
-            ["enter", "field", None, "Human", "[Pet]", None],
-            ["enter", "name", "pets", "Human", "[Pet]", None],
-            ["leave", "name", "pets", "Human", "[Pet]", None],
-            ["enter", "selection_set", None, "Pet", "[Pet]", None],
-            ["enter", "inline_fragment", None, "Pet", "Pet", None],
-            ["enter", "selection_set", None, "Pet", "Pet", None],
-            ["enter", "field", None, "Pet", "String", None],
-            ["enter", "name", "name", "Pet", "String", None],
-            ["leave", "name", "name", "Pet", "String", None],
-            ["leave", "field", None, "Pet", "String", None],
-            ["leave", "selection_set", None, "Pet", "Pet", None],
-            ["leave", "inline_fragment", None, "Pet", "Pet", None],
-            ["leave", "selection_set", None, "Pet", "[Pet]", None],
-            ["leave", "field", None, "Human", "[Pet]", None],
-            ["enter", "field", None, "Human", None, None],
-            ["enter", "name", "unknown", "Human", None, None],
-            ["leave", "name", "unknown", "Human", None, None],
-            ["leave", "field", None, "Human", None, None],
-            ["leave", "selection_set", None, "Human", "Human", None],
-            ["leave", "field", None, "QueryRoot", "Human", None],
-            ["leave", "selection_set", None, "QueryRoot", "QueryRoot", None],
-            ["leave", "operation_definition", None, None, "QueryRoot", None],
-            ["leave", "document", None, None, None, None],
-        ]
-
-    def maintains_type_info_during_edit():
-        visited = []
-        type_info = TypeInfo(test_schema)
-
-        ast = parse("{ human(id: 4) { name, pets }, alien }")
-
-        # noinspection PyMethodMayBeStatic
-        class TestVisitor(Visitor):
-            def enter(self, *args):
-                check_visitor_fn_args_edited(ast, *args)
-                parent_type = type_info.get_parent_type()
-                type_ = type_info.get_type()
-                input_type = type_info.get_input_type()
-                node = args[0]
-                visited.append(
-                    [
-                        "enter",
-                        node.kind,
-                        node.value if node.kind == "name" else None,
-                        str(parent_type) if parent_type else None,
-                        str(type_) if type_ else None,
-                        str(input_type) if input_type else None,
-                    ]
-                )
-
-                # Make a query valid by adding missing selection sets.
-                if (
-                    node.kind == "field"
-                    and not node.selection_set
-                    and is_composite_type(get_named_type(type_))
-                ):
-                    return FieldNode(
-                        alias=node.alias,
-                        name=node.name,
-                        arguments=node.arguments,
-                        directives=node.directives,
-                        selection_set=SelectionSetNode(
-                            selections=[FieldNode(name=NameNode(value="__typename"))]
-                        ),
-                    )
-
-            def leave(self, *args):
-                check_visitor_fn_args_edited(ast, *args)
-                parent_type = type_info.get_parent_type()
-                type_ = type_info.get_type()
-                input_type = type_info.get_input_type()
-                node = args[0]
-                visited.append(
-                    [
-                        "leave",
-                        node.kind,
-                        node.value if node.kind == "name" else None,
-                        str(parent_type) if parent_type else None,
-                        str(type_) if type_ else None,
-                        str(input_type) if input_type else None,
-                    ]
-                )
-
-        edited_ast = visit(ast, TypeInfoVisitor(type_info, TestVisitor()))
-
-        assert ast == parse("{ human(id: 4) { name, pets }, alien }")
-
-        assert print_ast(edited_ast) == print_ast(
-            parse(
-                "{ human(id: 4) { name, pets { __typename } },"
-                " alien { __typename } }"
-            )
-        )
-
-        assert visited == [
-            ["enter", "document", None, None, None, None],
-            ["enter", "operation_definition", None, None, "QueryRoot", None],
-            ["enter", "selection_set", None, "QueryRoot", "QueryRoot", None],
-            ["enter", "field", None, "QueryRoot", "Human", None],
-            ["enter", "name", "human", "QueryRoot", "Human", None],
-            ["leave", "name", "human", "QueryRoot", "Human", None],
-            ["enter", "argument", None, "QueryRoot", "Human", "ID"],
-            ["enter", "name", "id", "QueryRoot", "Human", "ID"],
-            ["leave", "name", "id", "QueryRoot", "Human", "ID"],
-            ["enter", "int_value", None, "QueryRoot", "Human", "ID"],
-            ["leave", "int_value", None, "QueryRoot", "Human", "ID"],
-            ["leave", "argument", None, "QueryRoot", "Human", "ID"],
-            ["enter", "selection_set", None, "Human", "Human", None],
-            ["enter", "field", None, "Human", "String", None],
-            ["enter", "name", "name", "Human", "String", None],
-            ["leave", "name", "name", "Human", "String", None],
-            ["leave", "field", None, "Human", "String", None],
-            ["enter", "field", None, "Human", "[Pet]", None],
-            ["enter", "name", "pets", "Human", "[Pet]", None],
-            ["leave", "name", "pets", "Human", "[Pet]", None],
-            ["enter", "selection_set", None, "Pet", "[Pet]", None],
-            ["enter", "field", None, "Pet", "String!", None],
-            ["enter", "name", "__typename", "Pet", "String!", None],
-            ["leave", "name", "__typename", "Pet", "String!", None],
-            ["leave", "field", None, "Pet", "String!", None],
-            ["leave", "selection_set", None, "Pet", "[Pet]", None],
-            ["leave", "field", None, "Human", "[Pet]", None],
-            ["leave", "selection_set", None, "Human", "Human", None],
-            ["leave", "field", None, "QueryRoot", "Human", None],
-            ["enter", "field", None, "QueryRoot", "Alien", None],
-            ["enter", "name", "alien", "QueryRoot", "Alien", None],
-            ["leave", "name", "alien", "QueryRoot", "Alien", None],
-            ["enter", "selection_set", None, "Alien", "Alien", None],
-            ["enter", "field", None, "Alien", "String!", None],
-            ["enter", "name", "__typename", "Alien", "String!", None],
-            ["leave", "name", "__typename", "Alien", "String!", None],
-            ["leave", "field", None, "Alien", "String!", None],
-            ["leave", "selection_set", None, "Alien", "Alien", None],
-            ["leave", "field", None, "QueryRoot", "Alien", None],
-            ["leave", "selection_set", None, "QueryRoot", "QueryRoot", None],
-            ["leave", "operation_definition", None, None, "QueryRoot", None],
-            ["leave", "document", None, None, None, None],
-        ]
