@@ -73,9 +73,26 @@ def email_schema_with_resolvers(subscribe_fn=None, resolve_fn=None):
 
 email_schema = email_schema_with_resolvers()
 
+default_subscription_ast = parse(
+    """
+    subscription ($priority: Int = 0) {
+      importantEmail(priority: $priority) {
+        email {
+          from
+          subject
+        }
+        inbox {
+          unread
+          total
+        }
+      }
+    }
+    """
+)
+
 
 async def create_subscription(
-    pubsub, schema: GraphQLSchema = email_schema, ast=None, variables=None
+    pubsub, schema: GraphQLSchema = email_schema, document=default_subscription_ast
 ):
     data: Dict[str, Any] = {
         "inbox": {
@@ -101,34 +118,17 @@ async def create_subscription(
             {"importantEmail": {"email": new_email, "inbox": data["inbox"]}},
         )
 
-    default_ast = parse(
-        """
-        subscription ($priority: Int = 0) {
-          importantEmail(priority: $priority) {
-            email {
-              from
-              subject
-            }
-            inbox {
-              unread
-              total
-            }
-          }
-        }
-        """
-    )
-
     # `subscribe` yields AsyncIterator or ExecutionResult
     return (
         send_important_email,
-        await subscribe(schema, ast or default_ast, data, variable_values=variables),
+        await subscribe(schema=schema, document=document, root_value=data),
     )
 
 
 # Check all error cases when initializing the subscription.
 def describe_subscription_initialization_phase():
     @mark.asyncio
-    async def accepts_keyword_arguments():
+    async def accepts_positional_arguments():
         document = parse(
             """
             subscription {
@@ -139,14 +139,16 @@ def describe_subscription_initialization_phase():
 
         async def empty_async_iterator(_info):
             for value in ():  # type: ignore
-                yield value
+                yield value  # pragma: no cover
 
-        await subscribe(
-            schema=email_schema,
-            document=document,
-            root_value={"importantEmail": empty_async_iterator},
+        ai = await subscribe(
+            email_schema, document, {"importantEmail": empty_async_iterator}
         )
 
+        with raises(StopAsyncIteration):
+            await anext(ai)
+        await ai.aclose()  # type: ignore
+
     @mark.asyncio
     async def accepts_multiple_subscription_fields_defined_in_schema():
         pubsub = EventEmitter()
@@ -196,16 +198,17 @@ def describe_subscription_initialization_phase():
             ),
         )
 
-        ast = parse(
-            """
+        subscription = await subscribe(
+            schema=schema,
+            document=parse(
+                """
             subscription {
               importantEmail
             }
             """
+            ),
         )
 
-        subscription = await subscribe(schema, ast)
-
         pubsub.emit("importantEmail", {"importantEmail": {}})
 
         await anext(subscription)
@@ -229,16 +232,17 @@ def describe_subscription_initialization_phase():
             ),
         )
 
-        ast = parse(
-            """
+        subscription = await subscribe(
+            schema=schema,
+            document=parse(
+                """
             subscription {
               importantEmail
             }
             """
+            ),
         )
 
-        subscription = await subscribe(schema, ast)
-
         pubsub.emit("importantEmail", {"importantEmail": {}})
 
         await anext(subscription)
@@ -251,7 +255,7 @@ def describe_subscription_initialization_phase():
             did_resolve["importantEmail"] = True
             return EventEmitterAsyncIterator(EventEmitter(), "event")
 
-        def subscribe_non_important(_inbox, _info):
+        def subscribe_non_important(_inbox, _info):  # pragma: no cover
             did_resolve["nonImportantEmail"] = True
             return EventEmitterAsyncIterator(EventEmitter(), "event")
 
@@ -267,20 +271,20 @@ def describe_subscription_initialization_phase():
             },
         )
 
-        test_schema = GraphQLSchema(
-            query=QueryType, subscription=SubscriptionTypeMultiple
-        )
+        schema = GraphQLSchema(query=QueryType, subscription=SubscriptionTypeMultiple)
 
-        ast = parse(
-            """
+        subscription = await subscribe(
+            schema=schema,
+            document=parse(
+                """
             subscription {
               importantEmail
               nonImportantEmail
             }
             """
+            ),
         )
 
-        subscription = await subscribe(test_schema, ast)
         ignored = anext(subscription)  # Ask for a result, but ignore it.
 
         assert did_resolve["importantEmail"] is True
@@ -321,7 +325,7 @@ def describe_subscription_initialization_phase():
             # noinspection PyTypeChecker
             await subscribe(email_schema, None)  # type: ignore
 
-        assert str(exc_info.value) == "Must provide document"
+        assert str(exc_info.value) == "Must provide document."
 
         with raises(TypeError, match="missing .* positional argument: 'document'"):
             # noinspection PyTypeChecker
@@ -339,7 +343,7 @@ def describe_subscription_initialization_phase():
 
         pubsub = EventEmitter()
 
-        subscription = (await create_subscription(pubsub, ast=ast))[1]
+        subscription = (await create_subscription(pubsub, document=ast))[1]
 
         assert subscription == (
             None,
@@ -351,6 +355,11 @@ def describe_subscription_initialization_phase():
             ],
         )
 
+    @mark.asyncio
+    async def should_pass_through_unexpected_errors_thrown_in_subscribe():
+        with raises(TypeError, match="Must provide document\\."):
+            await subscribe(schema=email_schema, document={})  # type: ignore
+
     @mark.asyncio
     async def throws_an_error_if_subscribe_does_not_return_an_iterator():
         invalid_email_schema = GraphQLSchema(
@@ -371,15 +380,15 @@ def describe_subscription_initialization_phase():
             await create_subscription(pubsub, invalid_email_schema)
 
         assert str(exc_info.value) == (
-            "Subscription field must return AsyncIterable. Received: 'test'"
+            "Subscription field must return AsyncIterable. Received: 'test'."
         )
 
     @mark.asyncio
     async def resolves_to_an_error_for_subscription_resolver_errors():
         async def test_reports_error(schema):
             result = await subscribe(
-                schema,
-                parse(
+                schema=schema,
+                document=parse(
                     """
                     subscription {
                       importantEmail
@@ -448,39 +457,22 @@ def describe_subscription_initialization_phase():
             """
         )
 
-        pubsub = EventEmitter()
-        data = {
-            "inbox": {
-                "emails": [
-                    {
-                        "from": "joe@graphql.org",
-                        "subject": "Hello",
-                        "message": "Hello World",
-                        "unread": False,
-                    }
-                ]
-            },
-            "importantEmail": lambda _info: EventEmitterAsyncIterator(
-                pubsub, "importantEmail"
-            ),
-        }
-
         result = await subscribe(
-            email_schema, ast, data, variable_values={"priority": "meow"}
+            schema=email_schema, document=ast, variable_values={"priority": "meow"},
         )
 
         assert result == (
             None,
             [
                 {
-                    "message": "Variable '$priority' got invalid value 'meow'; Expected"
-                    " type Int. Int cannot represent non-integer value: 'meow'",
+                    "message": "Variable '$priority' got invalid value 'meow';"
+                    " Int cannot represent non-integer value: 'meow'",
                     "locations": [(2, 27)],
                 }
             ],
         )
 
-        assert result.errors[0].original_error is not None  # type: ignore
+        assert result.errors[0].original_error is None  # type: ignore
 
 
 # Once a subscription returns a valid AsyncIterator, it can still yield errors.
@@ -864,8 +856,8 @@ def describe_subscription_publish_phase():
         erroring_email_schema = email_schema_with_resolvers(subscribe_fn, resolve_fn)
 
         subscription = await subscribe(
-            erroring_email_schema,
-            parse(
+            schema=erroring_email_schema,
+            document=parse(
                 """
                 subscription {
                   importantEmail {
@@ -900,8 +892,8 @@ def describe_subscription_publish_phase():
         async_email_schema = email_schema_with_resolvers(subscribe_fn, resolve_fn)
 
         subscription = await subscribe(
-            async_email_schema,
-            parse(
+            schema=async_email_schema,
+            document=parse(
                 """
                 subscription {
                   importantEmail {
