@@ -7,6 +7,7 @@ from graphql.type import (
     GraphQLBoolean,
     GraphQLEnumType,
     GraphQLField,
+    GraphQLFloat,
     GraphQLInputObjectType,
     GraphQLInt,
     GraphQLInterfaceType,
@@ -23,7 +24,12 @@ from graphql.type import (
     GraphQLDirective,
     assert_object_type,
 )
-from graphql.utilities import build_schema, print_schema, print_introspection_schema
+from graphql.utilities import (
+    build_schema,
+    print_schema,
+    print_introspection_schema,
+    print_value,
+)
 
 
 def print_for_test(schema: GraphQLSchema) -> str:
@@ -281,23 +287,62 @@ def describe_type_system_printer():
             """  # noqa: E501
         )
 
-    def prints_custom_query_root_type():
-        custom_query_type = GraphQLObjectType(
-            "CustomQueryType", {"bar": GraphQLField(GraphQLString)}
+    def prints_schema_with_description():
+        schema = GraphQLSchema(
+            description="Schema description.", query=GraphQLObjectType("Query", {})
         )
 
-        schema = GraphQLSchema(custom_query_type)
+        output = print_for_test(schema)
+        assert output == dedent(
+            '''
+            """Schema description."""
+            schema {
+              query: Query
+            }
+
+            type Query
+            '''
+        )
+
+    def prints_custom_query_root_types():
+        schema = GraphQLSchema(query=GraphQLObjectType("CustomType", {}))
 
         output = print_for_test(schema)
         assert output == dedent(
             """
             schema {
-              query: CustomQueryType
+              query: CustomType
             }
 
-            type CustomQueryType {
-              bar: String
+            type CustomType
+            """
+        )
+
+    def prints_custom_mutation_root_types():
+        schema = GraphQLSchema(mutation=GraphQLObjectType("CustomType", {}))
+
+        output = print_for_test(schema)
+        assert output == dedent(
+            """
+            schema {
+              mutation: CustomType
             }
+
+            type CustomType
+            """
+        )
+
+    def prints_custom_subscription_root_types():
+        schema = GraphQLSchema(subscription=GraphQLObjectType("CustomType", {}))
+
+        output = print_for_test(schema)
+        assert output == dedent(
+            """
+            schema {
+              subscription: CustomType
+            }
+
+            type CustomType
             """
         )
 
@@ -331,8 +376,8 @@ def describe_type_system_printer():
             name="Foo", fields={"str": GraphQLField(GraphQLString)}
         )
 
-        baaz_type = GraphQLInterfaceType(
-            name="Baaz", fields={"int": GraphQLField(GraphQLInt)}
+        baz_type = GraphQLInterfaceType(
+            name="Baz", fields={"int": GraphQLField(GraphQLInt)}
         )
 
         bar_type = GraphQLObjectType(
@@ -341,18 +386,14 @@ def describe_type_system_printer():
                 "str": GraphQLField(GraphQLString),
                 "int": GraphQLField(GraphQLInt),
             },
-            interfaces=[foo_type, baaz_type],
+            interfaces=[foo_type, baz_type],
         )
 
         schema = GraphQLSchema(types=[bar_type])
         output = print_for_test(schema)
         assert output == dedent(
             """
-            interface Baaz {
-              int: Int
-            }
-
-            type Bar implements Foo & Baaz {
+            type Bar implements Foo & Baz {
               str: String
               int: Int
             }
@@ -360,6 +401,59 @@ def describe_type_system_printer():
             interface Foo {
               str: String
             }
+
+            interface Baz {
+              int: Int
+            }
+            """
+        )
+
+    def prints_hierarchical_interface():
+        foo_type = GraphQLInterfaceType(
+            name="Foo", fields={"str": GraphQLField(GraphQLString)}
+        )
+
+        baz_type = GraphQLInterfaceType(
+            name="Baz",
+            interfaces=[foo_type],
+            fields={
+                "int": GraphQLField(GraphQLInt),
+                "str": GraphQLField(GraphQLString),
+            },
+        )
+
+        bar_type = GraphQLObjectType(
+            name="Bar",
+            fields={
+                "str": GraphQLField(GraphQLString),
+                "int": GraphQLField(GraphQLInt),
+            },
+            interfaces=[foo_type, baz_type],
+        )
+
+        query = GraphQLObjectType(name="Query", fields={"bar": GraphQLField(bar_type)})
+
+        schema = GraphQLSchema(query, types=[bar_type])
+        output = print_for_test(schema)
+        assert output == dedent(
+            """
+            type Bar implements Foo & Baz {
+              str: String
+              int: Int
+            }
+
+            interface Foo {
+              str: String
+            }
+
+            interface Baz implements Foo {
+              int: Int
+              str: String
+            }
+
+            type Query {
+              bar: Bar
+            }
             """
         )
 
@@ -382,9 +476,7 @@ def describe_type_system_printer():
         output = print_for_test(schema)
         assert output == dedent(
             """
-            type Bar {
-              str: String
-            }
+            union SingleUnion = Foo
 
             type Foo {
               bool: Boolean
@@ -392,7 +484,9 @@ def describe_type_system_printer():
 
             union MultipleUnion = Foo | Bar
 
-            union SingleUnion = Foo
+            type Bar {
+              str: String
+            }
             """
         )
 
@@ -491,6 +585,17 @@ def describe_type_system_printer():
             '''  # noqa: E501
         )
 
+    def prints_an_empty_description():
+        output = print_single_field_schema(GraphQLField(GraphQLString, description=""))
+        assert output == dedent(
+            '''
+            type Query {
+              """"""
+              singleField: String
+            }
+            '''
+        )
+
     def one_line_prints_a_short_description():
         description = "This field is awesome"
         output = print_single_field_schema(
@@ -512,7 +617,7 @@ def describe_type_system_printer():
     def prints_introspection_schema():
         schema = GraphQLSchema()
         output = print_introspection_schema(schema)
-        assert output == dedent(
+        introspection_schema = dedent(
             '''
             """
             Directs the executor to include this field or fragment only when the `if` argument is true.
@@ -533,31 +638,138 @@ def describe_type_system_printer():
             """Marks an element of a GraphQL schema as no longer supported."""
             directive @deprecated(
               """
-              Explains why this element was deprecated, usually also including a suggestion
-              for how to access supported similar data. Formatted using the Markdown syntax,
-              as specified by [CommonMark](https://commonmark.org/).
+              Explains why this element was deprecated, usually also including a suggestion for how to access supported similar data. Formatted using the Markdown syntax, as specified by [CommonMark](https://commonmark.org/).
               """
               reason: String = "No longer supported"
             ) on FIELD_DEFINITION | ENUM_VALUE
 
+            """
+            A GraphQL Schema defines the capabilities of a GraphQL server. It exposes all available types and directives on the server, as well as the entry points for query, mutation, and subscription operations.
+            """
+            type __Schema {
+              description: String
+
+              """A list of all types supported by this server."""
+              types: [__Type!]!
+
+              """The type that query operations will be rooted at."""
+              queryType: __Type!
+
+              """
+              If this server supports mutation, the type that mutation operations will be rooted at.
+              """
+              mutationType: __Type
+
+              """
+              If this server support subscription, the type that subscription operations will be rooted at.
+              """
+              subscriptionType: __Type
+
+              """A list of all directives supported by this server."""
+              directives: [__Directive!]!
+            }
+
+            """
+            The fundamental unit of any GraphQL Schema is the type. There are many kinds of types in GraphQL as represented by the `__TypeKind` enum.
+
+            Depending on the kind of a type, certain fields describe information about that type. Scalar types provide no information beyond a name and description, while Enum types provide their values. Object and Interface types provide the fields they describe. Abstract types, Union and Interface, provide the Object types possible at runtime. List and NonNull types compose other types.
+            """
+            type __Type {
+              kind: __TypeKind!
+              name: String
+              description: String
+              fields(includeDeprecated: Boolean = false): [__Field!]
+              interfaces: [__Type!]
+              possibleTypes: [__Type!]
+              enumValues(includeDeprecated: Boolean = false): [__EnumValue!]
+              inputFields: [__InputValue!]
+              ofType: __Type
+            }
+
+            """An enum describing what kind of type a given `__Type` is."""
+            enum __TypeKind {
+              """Indicates this type is a scalar."""
+              SCALAR
+
+              """
+              Indicates this type is an object. `fields` and `interfaces` are valid fields.
+              """
+              OBJECT
+
+              """
+              Indicates this type is an interface. `fields`, `interfaces`, and `possibleTypes` are valid fields.
+              """
+              INTERFACE
+
+              """Indicates this type is a union. `possibleTypes` is a valid field."""
+              UNION
+
+              """Indicates this type is an enum. `enumValues` is a valid field."""
+              ENUM
+
+              """
+              Indicates this type is an input object. `inputFields` is a valid field.
+              """
+              INPUT_OBJECT
+
+              """Indicates this type is a list. `ofType` is a valid field."""
+              LIST
+
+              """Indicates this type is a non-null. `ofType` is a valid field."""
+              NON_NULL
+            }
+
+            """
+            Object and Interface types are described by a list of Fields, each of which has a name, potentially a list of arguments, and a return type.
+            """
+            type __Field {
+              name: String!
+              description: String
+              args: [__InputValue!]!
+              type: __Type!
+              isDeprecated: Boolean!
+              deprecationReason: String
+            }
+
+            """
+            Arguments provided to Fields or Directives and the input fields of an InputObject are represented as Input Values which describe their type and optionally a default value.
+            """
+            type __InputValue {
+              name: String!
+              description: String
+              type: __Type!
+
+              """
+              A GraphQL-formatted string representing the default value for this input value.
+              """
+              defaultValue: String
+            }
+
+            """
+            One possible value for a given Enum. Enum values are unique values, not a placeholder for a string or numeric value. However an Enum value is returned in a JSON response as a string.
+            """
+            type __EnumValue {
+              name: String!
+              description: String
+              isDeprecated: Boolean!
+              deprecationReason: String
+            }
+
             """
             A Directive provides a way to describe alternate runtime execution and type validation behavior in a GraphQL document.
 
-            In some cases, you need to provide options to alter GraphQL's execution behavior
-            in ways field arguments will not suffice, such as conditionally including or
-            skipping a field. Directives provide this by describing additional information
-            to the executor.
+            In some cases, you need to provide options to alter GraphQL's execution behavior in ways field arguments will not suffice, such as conditionally including or skipping a field. Directives provide this by describing additional information to the executor.
             """
             type __Directive {
               name: String!
               description: String
+              isRepeatable: Boolean!
               locations: [__DirectiveLocation!]!
               args: [__InputValue!]!
             }
 
             """
-            A Directive can be adjacent to many parts of the GraphQL language, a
-            __DirectiveLocation describes one such possible adjacencies.
+            A Directive can be adjacent to many parts of the GraphQL language, a __DirectiveLocation describes one such possible adjacencies.
             """
             enum __DirectiveLocation {
               """Location adjacent to a query operation."""
@@ -617,127 +829,12 @@ def describe_type_system_printer():
               """Location adjacent to an input object field definition."""
               INPUT_FIELD_DEFINITION
             }
-
-            """
-            One possible value for a given Enum. Enum values are unique values, not a
-            placeholder for a string or numeric value. However an Enum value is returned in
-            a JSON response as a string.
-            """
-            type __EnumValue {
-              name: String!
-              description: String
-              isDeprecated: Boolean!
-              deprecationReason: String
-            }
-
-            """
-            Object and Interface types are described by a list of Fields, each of which has
-            a name, potentially a list of arguments, and a return type.
-            """
-            type __Field {
-              name: String!
-              description: String
-              args: [__InputValue!]!
-              type: __Type!
-              isDeprecated: Boolean!
-              deprecationReason: String
-            }
-
-            """
-            Arguments provided to Fields or Directives and the input fields of an
-            InputObject are represented as Input Values which describe their type and
-            optionally a default value.
-            """
-            type __InputValue {
-              name: String!
-              description: String
-              type: __Type!
-
-              """
-              A GraphQL-formatted string representing the default value for this input value.
-              """
-              defaultValue: String
-            }
-
-            """
-            A GraphQL Schema defines the capabilities of a GraphQL server. It exposes all
-            available types and directives on the server, as well as the entry points for
-            query, mutation, and subscription operations.
-            """
-            type __Schema {
-              """A list of all types supported by this server."""
-              types: [__Type!]!
-
-              """The type that query operations will be rooted at."""
-              queryType: __Type!
-
-              """
-              If this server supports mutation, the type that mutation operations will be rooted at.
-              """
-              mutationType: __Type
-
-              """
-              If this server support subscription, the type that subscription operations will be rooted at.
-              """
-              subscriptionType: __Type
-
-              """A list of all directives supported by this server."""
-              directives: [__Directive!]!
-            }
-
-            """
-            The fundamental unit of any GraphQL Schema is the type. There are many kinds of
-            types in GraphQL as represented by the `__TypeKind` enum.
-
-            Depending on the kind of a type, certain fields describe information about that
-            type. Scalar types provide no information beyond a name and description, while
-            Enum types provide their values. Object and Interface types provide the fields
-            they describe. Abstract types, Union and Interface, provide the Object types
-            possible at runtime. List and NonNull types compose other types.
-            """
-            type __Type {
-              kind: __TypeKind!
-              name: String
-              description: String
-              fields(includeDeprecated: Boolean = false): [__Field!]
-              interfaces: [__Type!]
-              possibleTypes: [__Type!]
-              enumValues(includeDeprecated: Boolean = false): [__EnumValue!]
-              inputFields: [__InputValue!]
-              ofType: __Type
-            }
-
-            """An enum describing what kind of type a given `__Type` is."""
-            enum __TypeKind {
-              """Indicates this type is a scalar."""
-              SCALAR
-
-              """
-              Indicates this type is an object. `fields` and `interfaces` are valid fields.
-              """
-              OBJECT
-
-              """
-              Indicates this type is an interface. `fields` and `possibleTypes` are valid fields.
-              """
-              INTERFACE
-
-              """Indicates this type is a union. `possibleTypes` is a valid field."""
-              UNION
-
-              """Indicates this type is an enum. `enumValues` is a valid field."""
-              ENUM
-
-              """
-              Indicates this type is an input object. `inputFields` is a valid field.
-              """
-              INPUT_OBJECT
-
-              """Indicates this type is a list. `ofType` is a valid field."""
-              LIST
-
-              """Indicates this type is a non-null. `ofType` is a valid field."""
-              NON_NULL
-            }
             '''  # noqa: E501
         )
+        assert output == introspection_schema
+
+
+def describe_print_value():
+    def print_value_convenience_function():
+        assert print_value(1.5, GraphQLFloat) == "1.5"
+        assert print_value("foo", GraphQLString) == '"foo"'
