@@ -1,12 +1,15 @@
 from functools import partial
 
-from graphql.validation import ValuesOfCorrectTypeRule
-from graphql.validation.rules.values_of_correct_type import (
-    bad_value_message,
-    bad_enum_value_message,
-    required_field_message,
-    unknown_field_message,
+from graphql.pyutils import Undefined
+from graphql.type import (
+    GraphQLArgument,
+    GraphQLField,
+    GraphQLObjectType,
+    GraphQLSchema,
+    GraphQLScalarType,
+    GraphQLString,
 )
+from graphql.validation import ValuesOfCorrectTypeRule
 
 from .harness import assert_validation_errors
 
@@ -15,34 +18,6 @@ assert_errors = partial(assert_validation_errors, ValuesOfCorrectTypeRule)
 assert_valid = partial(assert_errors, errors=[])
 
 
-def bad_value(type_name, value, line, column, message=None):
-    return {
-        "message": bad_value_message(type_name, value, message),
-        "locations": [(line, column)],
-    }
-
-
-def bad_enum_value(type_name, value, line, column, message=None):
-    return {
-        "message": bad_enum_value_message(type_name, value, message),
-        "locations": [(line, column)],
-    }
-
-
-def required_field(type_name, field_name, field_type_name, line, column):
-    return {
-        "message": required_field_message(type_name, field_name, field_type_name),
-        "locations": [(line, column)],
-    }
-
-
-def unknown_field(type_name, field_name, line, column, suggested_fields):
-    return {
-        "message": unknown_field_message(type_name, field_name, suggested_fields),
-        "locations": [(line, column)],
-    }
-
-
 def describe_validate_values_of_correct_type():
     def describe_valid_values():
         def good_int_value():
@@ -197,7 +172,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("String", "1", 4, 47)],
+                [
+                    {
+                        "message": "String cannot represent a non string value: 1",
+                        "locations": [(4, 47)],
+                    },
+                ],
             )
 
         def float_into_string():
@@ -209,7 +189,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("String", "1.0", 4, 47)],
+                [
+                    {
+                        "message": "String cannot represent a non string value: 1.0",
+                        "locations": [(4, 47)],
+                    },
+                ],
             )
 
         def boolean_into_string():
@@ -221,7 +206,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("String", "true", 4, 47)],
+                [
+                    {
+                        "message": "String cannot represent a non string value: true",
+                        "locations": [(4, 47)],
+                    },
+                ],
             )
 
         def unquoted_string_into_string():
@@ -233,7 +223,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("String", "BAR", 4, 47)],
+                [
+                    {
+                        "message": "String cannot represent a non string value: BAR",
+                        "locations": [(4, 47)],
+                    },
+                ],
             )
 
     def describe_invalid_int_values():
@@ -246,7 +241,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Int", '"3"', 4, 41)],
+                [
+                    {
+                        "message": 'Int cannot represent non-integer value: "3"',
+                        "locations": [(4, 41)],
+                    },
+                ],
             )
 
         def big_int_into_int():
@@ -258,7 +258,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Int", "829384293849283498239482938", 4, 41)],
+                [
+                    {
+                        "message": "Int cannot represent non 32-bit signed integer"
+                        " value: 829384293849283498239482938",
+                        "locations": [(4, 41)],
+                    },
+                ],
             )
 
         def unquoted_string_into_int():
@@ -270,7 +276,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Int", "FOO", 4, 41)],
+                [
+                    {
+                        "message": "Int cannot represent non-integer value: FOO",
+                        "locations": [(4, 41)],
+                    },
+                ],
             )
 
         def simple_float_into_int():
@@ -282,7 +293,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Int", "3.0", 4, 41)],
+                [
+                    {
+                        "message": "Int cannot represent non-integer value: 3.0",
+                        "locations": [(4, 41)],
+                    }
+                ],
             )
 
         def float_into_int():
@@ -294,7 +310,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Int", "3.333", 4, 41)],
+                [
+                    {
+                        "message": "Int cannot represent non-integer value: 3.333",
+                        "locations": [(4, 41)],
+                    },
+                ],
             )
 
     def describe_invalid_float_values():
@@ -307,7 +328,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Float", '"3.333"', 4, 45)],
+                [
+                    {
+                        "message": 'Float cannot represent non numeric value: "3.333"',
+                        "locations": [(4, 45)],
+                    },
+                ],
             )
 
         def boolean_into_float():
@@ -319,7 +345,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Float", "true", 4, 45)],
+                [
+                    {
+                        "message": "Float cannot represent non numeric value: true",
+                        "locations": [(4, 45)],
+                    },
+                ],
             )
 
         def unquoted_into_float():
@@ -331,7 +362,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Float", "FOO", 4, 45)],
+                [
+                    {
+                        "message": "Float cannot represent non numeric value: FOO",
+                        "locations": [(4, 45)],
+                    },
+                ],
             )
 
     def describe_invalid_boolean_value():
@@ -344,7 +380,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Boolean", "2", 4, 49)],
+                [
+                    {
+                        "message": "Boolean cannot represent a non boolean value: 2",
+                        "locations": [(4, 49)],
+                    },
+                ],
             )
 
         def float_into_boolean():
@@ -356,7 +397,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Boolean", "1.0", 4, 49)],
+                [
+                    {
+                        "message": "Boolean cannot represent a non boolean value: 1.0",
+                        "locations": [(4, 49)],
+                    }
+                ],
             )
 
         def string_into_boolean():
@@ -368,7 +414,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Boolean", '"true"', 4, 49)],
+                [
+                    {
+                        "message": "Boolean cannot represent a non boolean value:"
+                        ' "true"',
+                        "locations": [(4, 49)],
+                    }
+                ],
             )
 
         def unquoted_into_boolean():
@@ -380,7 +432,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Boolean", "TRUE", 4, 49)],
+                [
+                    {
+                        "message": "Boolean cannot represent a non boolean value: TRUE",
+                        "locations": [(4, 49)],
+                    },
+                ],
             )
 
     def describe_invalid_id_value():
@@ -393,7 +450,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("ID", "1.0", 4, 39)],
+                [
+                    {
+                        "message": "ID cannot represent a non-string"
+                        " and non-integer value: 1.0",
+                        "locations": [(4, 39)],
+                    }
+                ],
             )
 
         def boolean_into_id():
@@ -405,7 +468,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("ID", "true", 4, 39)],
+                [
+                    {
+                        "message": "ID cannot represent a non-string"
+                        " and non-integer value: true",
+                        "locations": [(4, 39)],
+                    },
+                ],
             )
 
         def unquoted_into_id():
@@ -417,7 +486,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("ID", "SOMETHING", 4, 39)],
+                [
+                    {
+                        "message": "ID cannot represent a non-string"
+                        " and non-integer value: SOMETHING",
+                        "locations": [(4, 39)],
+                    },
+                ],
             )
 
     def describe_invalid_enum_value():
@@ -430,7 +505,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("DogCommand", "2", 4, 49)],
+                [
+                    {
+                        "message": "Enum 'DogCommand' cannot represent non-enum value:"
+                        " 2.",
+                        "locations": [(4, 49)],
+                    },
+                ],
             )
 
         def float_into_enum():
@@ -442,7 +523,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("DogCommand", "1.0", 4, 49)],
+                [
+                    {
+                        "message": "Enum 'DogCommand' cannot represent non-enum value:"
+                        " 1.0.",
+                        "locations": [(4, 49)],
+                    },
+                ],
             )
 
         def string_into_enum():
@@ -454,7 +541,14 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_enum_value("DogCommand", '"SIT"', 4, 49, ["SIT"])],
+                [
+                    {
+                        "message": "Enum 'DogCommand' cannot represent non-enum value:"
+                        ' "SIT".'
+                        " Did you mean the enum value 'SIT'?",
+                        "locations": [(4, 49)],
+                    },
+                ],
             )
 
         def boolean_into_enum():
@@ -466,7 +560,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("DogCommand", "true", 4, 49)],
+                [
+                    {
+                        "message": "Enum 'DogCommand' cannot represent non-enum value:"
+                        " true.",
+                        "locations": [(4, 49)],
+                    },
+                ],
             )
 
         def unknown_enum_value_into_enum():
@@ -478,7 +578,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("DogCommand", "JUGGLE", 4, 49)],
+                [
+                    {
+                        "message": "Value 'JUGGLE'"
+                        " does not exist in 'DogCommand' enum.",
+                        "locations": [(4, 49)],
+                    },
+                ],
             )
 
         def different_case_enum_value_into_enum():
@@ -490,7 +596,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_enum_value("DogCommand", "sit", 4, 49, ["SIT"])],
+                [
+                    {
+                        "message": "Value 'sit' does not exist in 'DogCommand' enum."
+                        " Did you mean the enum value 'SIT'?",
+                        "locations": [(4, 49)],
+                    },
+                ],
             )
 
     def describe_valid_list_value():
@@ -548,7 +660,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("String", "2", 4, 63)],
+                [
+                    {
+                        "message": "String cannot represent a non string value: 2",
+                        "locations": [(4, 63)],
+                    },
+                ],
             )
 
         def single_value_of_incorrect_type():
@@ -560,7 +677,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("[String]", "1", 4, 55)],
+                [
+                    {
+                        "message": "String cannot represent a non string value: 1",
+                        "locations": [(4, 55)],
+                    },
+                ],
             )
 
     def describe_valid_non_nullable_value():
@@ -569,7 +691,7 @@ def describe_validate_values_of_correct_type():
                 """
                 {
                   dog {
-                    isHousetrained(atOtherHomes: true)
+                    isHouseTrained(atOtherHomes: true)
                   }
                 }
                 """
@@ -580,7 +702,7 @@ def describe_validate_values_of_correct_type():
                 """
                 {
                   dog {
-                    isHousetrained
+                    isHouseTrained
                   }
                 }
                 """
@@ -641,7 +763,7 @@ def describe_validate_values_of_correct_type():
                 """
             )
 
-        def multiple_reqs_on_mixed_list():
+        def multiple_required_args_on_mixed_list():
             assert_valid(
                 """
                 {
@@ -652,7 +774,7 @@ def describe_validate_values_of_correct_type():
                 """
             )
 
-        def multiple_reqs_and_one_opt_on_mixed_list():
+        def multiple_required_and_one_optional_arg_on_mixed_list():
             assert_valid(
                 """
                 {
@@ -663,7 +785,7 @@ def describe_validate_values_of_correct_type():
                 """
             )
 
-        def all_reqs_and_and_opts_on_mixed_list():
+        def all_required_and_optional_args_on_mixed_list():
             assert_valid(
                 """
                 {
@@ -684,7 +806,16 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Int!", '"two"', 4, 40), bad_value("Int!", '"one"', 4, 53)],
+                [
+                    {
+                        "message": 'Int cannot represent non-integer value: "two"',
+                        "locations": [(4, 40)],
+                    },
+                    {
+                        "message": 'Int cannot represent non-integer value: "one"',
+                        "locations": [(4, 53)],
+                    },
+                ],
             )
 
         def incorrect_value_and_missing_argument_provided_required_arguments():
@@ -696,7 +827,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Int!", '"one"', 4, 40)],
+                [
+                    {
+                        "message": 'Int cannot represent non-integer value: "one"',
+                        "locations": [(4, 40)],
+                    },
+                ],
             )
 
         def null_value():
@@ -708,7 +844,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Int!", "null", 4, 40)],
+                [
+                    {
+                        "message": "Expected value of type 'Int!', found null.",
+                        "locations": [(4, 40)],
+                    },
+                ],
             )
 
     def describe_valid_input_object_value():
@@ -734,7 +875,7 @@ def describe_validate_values_of_correct_type():
                 """
             )
 
-        def partial_object_required_field_can_be_falsey():
+        def partial_object_required_field_can_be_falsy():
             assert_valid(
                 """
                 {
@@ -800,7 +941,13 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [required_field("ComplexInput", "requiredField", "Boolean!", 4, 49)],
+                [
+                    {
+                        "message": "Field 'ComplexInput.requiredField'"
+                        " of required type 'Boolean!' was not provided.",
+                        "locations": [(4, 49)],
+                    },
+                ],
             )
 
         def partial_object_invalid_field_type():
@@ -815,7 +962,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("String", "2", 5, 48)],
+                [
+                    {
+                        "message": "String cannot represent a non string value: 2",
+                        "locations": [(5, 48)],
+                    },
+                ],
             )
 
         def partial_object_null_to_non_null_field():
@@ -830,7 +982,12 @@ def describe_validate_values_of_correct_type():
                   }
                 }
                 """,
-                [bad_value("Boolean!", "null", 6, 37)],
+                [
+                    {
+                        "message": "Expected value of type 'Boolean!', found null.",
+                        "locations": [(6, 37)],
+                    }
+                ],
             )
 
         def partial_object_unknown_field_arg():
@@ -840,19 +997,18 @@ def describe_validate_values_of_correct_type():
                   complicatedArgs {
                     complexArgField(complexArg: {
                       requiredField: true,
-                      unknownField: "value"
+                      invalidField: "value"
                     })
                   }
                 }
                 """,
                 [
-                    unknown_field(
-                        "ComplexInput",
-                        "unknownField",
-                        6,
-                        23,
-                        ["nonNullField", "intField", "booleanField"],
-                    )
+                    {
+                        "message": "Field 'invalidField'"
+                        " is not defined by type 'ComplexInput'."
+                        " Did you mean 'intField'?",
+                        "locations": [(6, 23)],
+                    },
                 ],
             )
 
@@ -864,15 +1020,48 @@ def describe_validate_values_of_correct_type():
                 }
                 """,
                 [
-                    bad_value(
-                        "Invalid", "123", 3, 35, "Invalid scalar is always invalid: 123"
-                    )
+                    {
+                        "message": "Expected value of type 'Invalid', found 123;"
+                        " Invalid scalar is always invalid: 123",
+                        "locations": [(3, 35)],
+                    },
                 ],
             )
             assert str(errors[0].original_error) == (
                 "Invalid scalar is always invalid: 123"
             )
 
+        def reports_error_for_custom_scalar_that_returns_undefined():
+            custom_scalar = GraphQLScalarType(
+                "CustomScalar", parse_value=lambda value: Undefined
+            )
+
+            schema = GraphQLSchema(
+                GraphQLObjectType(
+                    "Query",
+                    {
+                        "invalidArg": GraphQLField(
+                            GraphQLString, args={"arg": GraphQLArgument(custom_scalar)}
+                        )
+                    },
+                )
+            )
+
+            assert_errors(
+                """
+                {
+                  invalidArg(arg: 123)
+                }
+                """,
+                [
+                    {
+                        "message": "Expected value of type 'CustomScalar', found 123.",
+                        "locations": [(3, 35)],
+                    },
+                ],
+                schema=schema,
+            )
+
         def allows_custom_scalar_to_accept_complex_literals():
             assert_valid(
                 """
@@ -910,8 +1099,15 @@ def describe_validate_values_of_correct_type():
                 }
                 """,
                 [
-                    bad_value("Boolean!", '"yes"', 3, 36),
-                    bad_value("Boolean!", "ENUM", 4, 36),
+                    {
+                        "message": "Boolean cannot represent a non boolean value:"
+                        ' "yes"',
+                        "locations": [(3, 36)],
+                    },
+                    {
+                        "message": "Boolean cannot represent a non boolean value: ENUM",
+                        "locations": [(4, 36)],
+                    },
                 ],
             )
 
@@ -955,9 +1151,18 @@ def describe_validate_values_of_correct_type():
                 }
                 """,
                 [
-                    bad_value("Int!", "null", 3, 30),
-                    bad_value("String!", "null", 4, 33),
-                    bad_value("Boolean!", "null", 5, 55),
+                    {
+                        "message": "Expected value of type 'Int!', found null.",
+                        "locations": [(3, 30)],
+                    },
+                    {
+                        "message": "Expected value of type 'String!', found null.",
+                        "locations": [(4, 33)],
+                    },
+                    {
+                        "message": "Expected value of type 'Boolean!', found null.",
+                        "locations": [(5, 55)],
+                    },
                 ],
             )
 
@@ -967,15 +1172,25 @@ def describe_validate_values_of_correct_type():
                 query InvalidDefaultValues(
                   $a: Int = "one",
                   $b: String = 4,
-                  $c: ComplexInput = "notverycomplex"
+                  $c: ComplexInput = "NotVeryComplex"
                 ) {
                   dog { name }
                 }
                 """,
                 [
-                    bad_value("Int", '"one"', 3, 29),
-                    bad_value("String", "4", 4, 32),
-                    bad_value("ComplexInput", '"notverycomplex"', 5, 38),
+                    {
+                        "message": 'Int cannot represent non-integer value: "one"',
+                        "locations": [(3, 29)],
+                    },
+                    {
+                        "message": "String cannot represent a non string value: 4",
+                        "locations": [(4, 32)],
+                    },
+                    {
+                        "message": "Expected value of type 'ComplexInput',"
+                        ' found "NotVeryComplex".',
+                        "locations": [(5, 38)],
+                    },
                 ],
             )
 
@@ -988,7 +1203,16 @@ def describe_validate_values_of_correct_type():
                   dog { name }
                 }
                 """,
-                [bad_value("Boolean!", "123", 3, 55), bad_value("Int", '"abc"', 3, 70)],
+                [
+                    {
+                        "message": "Boolean cannot represent a non boolean value: 123",
+                        "locations": [(3, 55)],
+                    },
+                    {
+                        "message": 'Int cannot represent non-integer value: "abc"',
+                        "locations": [(3, 70)],
+                    },
+                ],
             )
 
         def complex_variables_missing_required_fields():
@@ -998,7 +1222,13 @@ def describe_validate_values_of_correct_type():
                   dog { name }
                 }
                 """,
-                [required_field("ComplexInput", "requiredField", "Boolean!", 2, 63)],
+                [
+                    {
+                        "message": "Field 'ComplexInput.requiredField'"
+                        " of required type 'Boolean!' was not provided.",
+                        "locations": [(2, 63)],
+                    },
+                ],
             )
 
         def list_variables_with_invalid_item():
@@ -1008,5 +1238,10 @@ def describe_validate_values_of_correct_type():
                   dog { name }
                 }
                 """,
-                [bad_value("String", "2", 2, 58)],
+                [
+                    {
+                        "message": "String cannot represent a non string value: 2",
+                        "locations": [(2, 58)],
+                    },
+                ],
             )
